package ch.unisg.executorrobot.executor.adapter.out;
import java.io.IOException;
import java.io.StringReader;
import java.net.URI;
import java.net.http.HttpClient;
import java.net.http.HttpRequest;
import java.net.http.HttpResponse;
import java.util.HashMap;
import java.util.Map;
import java.util.Optional;
import java.util.concurrent.TimeUnit;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import ch.unisg.ics.interactions.wot.td.ThingDescription;
import ch.unisg.ics.interactions.wot.td.affordances.ActionAffordance;
import ch.unisg.ics.interactions.wot.td.affordances.Form;
import ch.unisg.ics.interactions.wot.td.clients.TDHttpRequest;
import ch.unisg.ics.interactions.wot.td.clients.TDHttpResponse;
import ch.unisg.ics.interactions.wot.td.io.TDGraphReader;
import ch.unisg.ics.interactions.wot.td.schemas.DataSchema;
import ch.unisg.ics.interactions.wot.td.schemas.ObjectSchema;
import ch.unisg.ics.interactions.wot.td.security.APIKeySecurityScheme;
import ch.unisg.ics.interactions.wot.td.security.SecurityScheme;
import ch.unisg.ics.interactions.wot.td.vocabularies.TD;
import ch.unisg.ics.interactions.wot.td.vocabularies.WoTSec;

import org.springframework.boot.autoconfigure.sendgrid.SendGridAutoConfiguration;
import org.springframework.context.annotation.Primary;
import org.springframework.stereotype.Component;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import ch.unisg.executorrobot.executor.application.port.out.UserToRobotPort;

@Component
@Primary
public class UserToRobotAdapter implements UserToRobotPort {

    @Override
    public String userToRobot() {

        String endpoint = "https://api.interactions.ics.unisg.ch/search/searchEngine";

        String input = "@prefix dct: <http://purl.org/dc/terms/> . select ?title where { ?title dct:title 'leubot1' }";

        var httpRequest = HttpRequest.newBuilder()
            .uri(URI.create(endpoint))
            .header("Content-Type", "application/json")
            .POST(HttpRequest.BodyPublishers.ofString(input))
            .build();

        var client = HttpClient.newHttpClient();

        try {
            String description = client.send(httpRequest, HttpResponse.BodyHandlers.ofString()).body();
            String leubot1Uri = null;

            DocumentBuilderFactory dbFactory = DocumentBuilderFactory.newInstance();
            DocumentBuilder dBuilder = dbFactory.newDocumentBuilder();
            Document doc = dBuilder.parse(new InputSource(new StringReader(description)));

            doc.getDocumentElement().normalize();

            NodeList results = doc.getElementsByTagName("uri");
            for (int temp = 0; temp < results.getLength(); temp += 1) {
                Node nNode = results.item(temp);
                if (nNode.getTextContent().contains("leubot1")) {
                    leubot1Uri = nNode.getTextContent();
                }
            }
            if (leubot1Uri == null) {
                // TODO implement logic if execution failed
                return "ERROR";
            }

            // Parse a TD from a string
            ThingDescription td = TDGraphReader.readFromURL(ThingDescription.TDFormat.RDF_TURTLE, leubot1Uri);

            String apiUrl = getAPIKey(td);
            System.out.println("FOUND API URL " + apiUrl);

            String apiKey = apiUrl.split("/")[apiUrl.split("/").length-1].split("]")[0];
            System.out.println("FOUND KEY " + apiKey);

            if(apiKey == null) {
                // TODO implement logic if execution failed
                return "ERROR";
            }

            try {
                TimeUnit.MILLISECONDS.sleep(1500);
            } catch (InterruptedException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }

            if (!moveRobot(td, apiKey)) {
                return "ERROR";
            }

            try {
                TimeUnit.MILLISECONDS.sleep(1500);
            } catch (InterruptedException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }

            delteUserFromRobot(apiUrl);

        } catch (IOException e) {
            e.printStackTrace();
        } catch (InterruptedException e) {
            e.printStackTrace();
        } catch (SAXException e1) {
            e1.printStackTrace();
        } catch (ParserConfigurationException e1) {
            e1.printStackTrace();
        }
        return "OK";
    }


    private String getAPIKey(ThingDescription td) {
        // Create the payload to be sent with the HTTP request
        Map<String, Object> logInPayload = new HashMap<>();
        logInPayload.put("http://xmlns.com/foaf/0.1/Name", "keanu rahimian");
        logInPayload.put("http://xmlns.com/foaf/0.1/Mbox", "keanu.rahimian@student.unisg.ch");

        // Get the affordance "Log-In" from the TD
        Optional<ActionAffordance> action = td.getActionByName("logIn");

        // Get the first form
        if (action.isEmpty()) {
            // TODO implement logic if execution failed
            return null;
        }

        Optional<Form> form = action.get().getFirstFormForOperationType(TD.invokeAction);

        if (form.isEmpty()) {
            // TODO implement logic if execution failed
            return null;
        }

        // If a form is found, use it to create and execute the HTTP request
        TDHttpRequest request = new TDHttpRequest(form.get(), TD.invokeAction);

        // Retrieve the input data schema from the action affordance
        Optional<DataSchema> inputSchema = action.get().getInputSchema();

        if(inputSchema.isPresent()) {
            request.setObjectPayload((ObjectSchema) inputSchema.get(), logInPayload);
        }

        try {
            TDHttpResponse response = request.execute();
            System.out.println("Received response with status code: " + response.getStatusCode());

            String url = response.getHeaders().get("Location");
            return url;

        } catch (IOException e) {
            e.printStackTrace();
        }

        return null;

    }

    private boolean moveRobot(ThingDescription td, String key) {

        // Create the payload to be sent with the Http request
        Map<String, Object> elbowPayload = new HashMap<>();
        elbowPayload.put("value", 400);

        // Get the affordance "setElbow" from the TD
        Optional<ActionAffordance> action = td.getActionByName("setElbow");

        // Get the first form
        if (action.isEmpty()) {
            return false;
        }

        Optional<Form> form = action.get().getFirstFormForOperationType(TD.invokeAction);

        if (form.isEmpty()) {
            // TODO implement logic if execution failed
            return false;
        }

        // Retrieve the input data schema from the action affordance
        Optional<DataSchema> inputSchema = action.get().getInputSchema();

        TDHttpRequest request = new TDHttpRequest(form.get(), TD.invokeAction);

        if(inputSchema.isPresent()) {
            request.setObjectPayload((ObjectSchema) inputSchema.get(), elbowPayload);
        }

        Optional<SecurityScheme> securityScheme = td.getFirstSecuritySchemeByType(WoTSec.APIKeySecurityScheme);
        if (securityScheme.isPresent()) {
            request.setAPIKey((APIKeySecurityScheme) securityScheme.get(), key);
        }

        try {
            TDHttpResponse response = request.execute();
            System.out.println("Received response with status code: " + response.getStatusCode());
            return true;
        } catch (IOException e) {
            e.printStackTrace();
        }

        return false;
    }

    private void delteUserFromRobot(String apiUrl) {
        HttpRequest request = HttpRequest.newBuilder()
        .uri(URI.create(apiUrl))
        .header("Content-Type", "application/json")
        .DELETE()
        .build();

        HttpClient client = HttpClient.newHttpClient();

        try {
            var response = client.send(request, HttpResponse.BodyHandlers.ofString());
            System.out.println(response.statusCode());
        } catch (IOException e) {
            e.printStackTrace();
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
    }

}
