package ch.unisg.roster.roster.adapter.in.web;

import java.util.logging.Logger;

import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import ch.unisg.roster.roster.application.port.in.ApplyForTaskCommand;
import ch.unisg.roster.roster.application.port.in.ApplyForTaskUseCase;
import ch.unisg.roster.roster.domain.ExecutorInfo;
import ch.unisg.roster.roster.domain.Task;
import org.springframework.web.server.ResponseStatusException;

@RestController
public class ApplyForTaskWebController {
    private final ApplyForTaskUseCase applyForTaskUseCase;

    public ApplyForTaskWebController(ApplyForTaskUseCase applyForTaskUseCase) {
        this.applyForTaskUseCase = applyForTaskUseCase;
    }

    Logger logger = Logger.getLogger(ApplyForTaskWebController.class.getName());

    /**
    *   Checks if task is available for the requesting executor.
    *   @return a task or 404 if no task available
    **/
    @PostMapping(path = "/task/apply", consumes = {"application/json"})
    public Task applyForTask (@RequestBody ExecutorInfo executorInfo) {
        logger.info("Roster | Execuor applying for task");
        ApplyForTaskCommand command = new ApplyForTaskCommand(executorInfo.getExecutorType(),
            executorInfo.getExecutorURI());

        Task task =  applyForTaskUseCase.applyForTask(command);

        if (task == null) {
            throw new ResponseStatusException(HttpStatus.NOT_FOUND);
        }

        return task;
    }
}
