package ch.unisg.roster.roster.adapter.in.web;

import java.util.logging.Logger;

import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RestController;

import ch.unisg.roster.roster.application.port.in.DeleteTaskCommand;
import ch.unisg.roster.roster.application.port.in.DeleteTaskUseCase;

@RestController
public class DeleteTaskController {
    private final DeleteTaskUseCase deleteTaskUseCase;

    public DeleteTaskController(DeleteTaskUseCase deleteTaskUseCase) {
        this.deleteTaskUseCase = deleteTaskUseCase;
    }

    Logger logger = Logger.getLogger(DeleteTaskController.class.getName());

    /**
    *   Controller to delete a task
    *   @return 200 OK, 409 Conflict
    **/
    @DeleteMapping(path = "/task/{taskId}")
    public ResponseEntity<Void> deleteTask(@PathVariable("taskId") String taskId) {
        logger.info("Roster | Delete task request.");
        DeleteTaskCommand command = new DeleteTaskCommand(taskId);

        if (deleteTaskUseCase.deleteTask(command)) {
            return new ResponseEntity<>(HttpStatus.OK);
        }
        return new ResponseEntity<>(HttpStatus.CONFLICT);
    }
}
