package ch.unisg.roster.roster.adapter.in.web;

import java.util.logging.Logger;

import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestHeader;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.servlet.function.ServerRequest.Headers;

import ch.unisg.roster.roster.application.port.in.NewTaskCommand;
import ch.unisg.roster.roster.application.port.in.NewTaskUseCase;
import ch.unisg.roster.roster.domain.Task;

@RestController
public class NewTaskController {
    private final NewTaskUseCase newTaskUseCase;

    public NewTaskController(NewTaskUseCase newTaskUseCase) {
        this.newTaskUseCase = newTaskUseCase;
    }

    Logger logger = Logger.getLogger(NewTaskController.class.getName());

    /**
    *   Controller which handles the new task event from the tasklist
    *   @return 201 Create or 409 Conflict
    **/
    @PostMapping(path = "/task", consumes = {"application/task+json"})
    public ResponseEntity<Void> newTaskController(@RequestBody Task task, @RequestHeader HttpHeaders header) {

            logger.info("Roster | New task with id:" + task.getTaskID());

            // Check if task URI is passed in body. (This happends when we handeling an external task) Else get task URI from header.
            String taskUri = task.getTaskUri();
            if (taskUri == null) {
                if (header.containsKey("Link")) {
                    for (String location : header.get("Link")) {
                        if (location.contains("rel=\"task\"")) {
                            taskUri = location.split(">")[0].substring(1);
                        }
                    }
                }
            }

            if (taskUri == null) {
                logger.warning("Roster | TaskUri still null, something went wrong!");
                return new ResponseEntity<>(HttpStatus.CONFLICT);
            }

            NewTaskCommand command = new NewTaskCommand(task.getTaskID(), taskUri, task.getTaskType(),
                task.getInputData());

            boolean success = newTaskUseCase.addNewTaskToQueue(command);

            logger.info("Roster | Could create task: " + success);

            if (success) {
                return new ResponseEntity<>(HttpStatus.CREATED);
            }
            return new ResponseEntity<>(HttpStatus.CONFLICT);

    }
}
