package ch.unisg.roster.roster.adapter.out.web;

import java.io.IOException;
import java.net.URI;
import java.net.http.HttpClient;
import java.net.http.HttpRequest;
import java.net.http.HttpResponse;
import java.util.HashMap;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Primary;
import org.springframework.stereotype.Component;

import ch.unisg.roster.roster.application.port.out.LaunchAuctionEventPort;
import ch.unisg.roster.roster.domain.event.LaunchAuctionEvent;

@Component
@Primary
public class LaunchAuctionEventAdapter implements LaunchAuctionEventPort {

    @Value("${auction.house.uri}")
    String server;

    Logger logger = Logger.getLogger(LaunchAuctionEventAdapter.class.getName());

    @Override
    public void launchAuctionEvent(LaunchAuctionEvent launchAuctionEvent) {

        var values = new HashMap<String, String>();

        values.put("taskUri", launchAuctionEvent.taskUri);
        values.put("taskType", launchAuctionEvent.taskType.getValue());

        var objectMapper = new ObjectMapper();
        String requestBody = null;
        try {
            requestBody = objectMapper.writeValueAsString(values);
        } catch (JsonProcessingException e) {
            e.printStackTrace();
        }

        HttpClient client = HttpClient.newHttpClient();
        HttpRequest request = HttpRequest.newBuilder()
            .uri(URI.create(server + "/auctions/"))
            .header("Content-Type", "application/auction+json")
            .POST(HttpRequest.BodyPublishers.ofString(requestBody))
            .build();

        try {
            client.send(request, HttpResponse.BodyHandlers.ofString());
        } catch (IOException e) {
            logger.log(Level.SEVERE, e.getLocalizedMessage(), e);
        } catch (InterruptedException e) {
            logger.log(Level.SEVERE, e.getLocalizedMessage(), e);
            Thread.currentThread().interrupt();
        }

    }

}
