package ch.unisg.roster.roster.application.service;

import java.util.logging.Logger;

import javax.transaction.Transactional;

import org.springframework.stereotype.Component;

import ch.unisg.roster.roster.application.port.in.NewTaskCommand;
import ch.unisg.roster.roster.application.port.in.NewTaskUseCase;
import ch.unisg.roster.roster.application.port.out.LaunchAuctionEventPort;
import ch.unisg.roster.roster.application.port.out.NewTaskEventPort;
import ch.unisg.roster.roster.domain.ExecutorRegistry;
import ch.unisg.roster.roster.domain.Roster;
import ch.unisg.roster.roster.domain.Task;
import ch.unisg.roster.roster.domain.event.LaunchAuctionEvent;
import ch.unisg.roster.roster.domain.event.NewTaskEvent;
import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor
@Component
@Transactional
public class NewTaskService implements NewTaskUseCase {

    private final NewTaskEventPort newTaskEventPort;

    private final LaunchAuctionEventPort launchAuctionEventPort;

    Logger logger = Logger.getLogger(NewTaskService.class.getName());

    /**
    *   Checks if we can execute the give task, if yes the task gets added to the task queue and return true.
    *   If the task can not be executed by an internal or auction house executor, the method return false.
    *   @return boolean
    **/
    @Override
    public boolean addNewTaskToQueue(NewTaskCommand command) {

        ExecutorRegistry executorRegistry = ExecutorRegistry.getInstance();

        if (!executorRegistry.containsTaskType(command.getTaskType())) {
            logger.info("Roster | Task with type " + command.getTaskType() + "can not be handled internaly. Send to auction house!");
            LaunchAuctionEvent launchAuctionEvent = new LaunchAuctionEvent( command.getTaskUri(),
            command.getTaskType());
            launchAuctionEventPort.launchAuctionEvent(launchAuctionEvent);
            return true;
        }

        logger.info("Roster | Task with type " + command.getTaskType() + " can be handled internaly.");

        Task task = new Task(command.getTaskID(), command.getTaskType(), command.getInputData());

        Roster.getInstance().addTaskToQueue(task);

        NewTaskEvent newTaskEvent = new NewTaskEvent(task.getTaskType());
        newTaskEventPort.publishNewTaskEvent(newTaskEvent);

        return true;
    }

}
