package ch.unisg.tapas.auctionhouse.adapter.in.messaging.websub;

import ch.unisg.tapas.auctionhouse.adapter.common.formats.AuctionJsonRepresentation;
import ch.unisg.tapas.auctionhouse.adapter.common.formats.BidJsonRepresentation;
import ch.unisg.tapas.auctionhouse.application.handler.AuctionStartedHandler;
import ch.unisg.tapas.auctionhouse.application.port.in.AuctionStartedEvent;
import ch.unisg.tapas.auctionhouse.domain.Auction;
import ch.unisg.tapas.auctionhouse.domain.Bid;
import ch.unisg.tapas.auctionhouse.domain.Auction.AuctionDeadline;
import ch.unisg.tapas.auctionhouse.domain.Auction.AuctionHouseUri;
import ch.unisg.tapas.auctionhouse.domain.Auction.AuctionId;
import ch.unisg.tapas.auctionhouse.domain.Auction.AuctionedTaskType;
import ch.unisg.tapas.auctionhouse.domain.Auction.AuctionedTaskUri;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.http.HttpClient;
import java.net.http.HttpRequest;
import java.net.http.HttpResponse;
import java.security.Timestamp;
import java.util.Collection;
import java.util.Date;

import com.fasterxml.jackson.core.JsonProcessingException;

import org.json.JSONArray;
import org.json.JSONObject;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

/**
 * This class is a template for handling auction started events received via WebSub
 */
@RestController
public class AuctionStartedEventListenerWebSubAdapter {
    private final AuctionStartedHandler auctionStartedHandler;

    public AuctionStartedEventListenerWebSubAdapter(AuctionStartedHandler auctionStartedHandler) {
        this.auctionStartedHandler = auctionStartedHandler;
    }
    /**
    *   Controller which listens to auction-started callbacks
    *   @return 200 OK
     * @throws URISyntaxException
    **/
    // TODO generate a new capability ID instead of using a hardcoded one.
    @PostMapping(path = "/auction-started/74c72c7f-2739-4124-aa35-a3225171a97c")
    public ResponseEntity<Void> handleExecutorAddedEvent(@RequestBody String payload) throws URISyntaxException {

        System.out.println("new auctions :O");
        System.out.println(payload);


        JSONArray auctions = new JSONArray(payload);
        if (auctions.length() > 0) {
            JSONObject auction = auctions.getJSONObject(0);
            System.out.print(auction);
            // try {
            //     System.out.println(auction.getString("deadline"));
            //     System.out.println(AuctionJsonRepresentation.deserialize(auction.toString()));

            //     auctionStartedHandler.handleAuctionStartedEvent(
            //         new AuctionStartedEvent(AuctionJsonRepresentation.deserialize(auction.toString())));
            // } catch (JsonProcessingException e) {
            //     // TODO Auto-generated catch block
            //     e.printStackTrace();
            // }


        String auctionHouseURI = "https://tapas-auction-house.86-119-35-40.nip.io/";
        // String auctionHouseURI = "http://b311-130-82-247-153.eu.ngrok.io";

        String taskListURI = "https://tapas-tasks.86-119-35-40.nip.io";
        // String taskListURI = "http://c64f-130-82-247-153.ngrok.io";

        // TODO Sanitize URIs
        String auctionId = auction.getString("auctionId");
        String auctionHouseUri = auction.getString("auctionHouseUri");
        String taskUri = auction.getString("taskUri");
        String taskType = auction.getString("taskType");
        String deadline = auction.getString("deadline");

            var bid = new Bid(
                    new Auction.AuctionId(auctionId),
                    new Bid.BidderName("Group-1"),
                    new Bid.BidderAuctionHouseUri(URI.create(auctionHouseURI)),
                    new Bid.BidderTaskListUri(URI.create(taskListURI))
                );

                String body;
                try {
                    body = BidJsonRepresentation.serialize(bid);
                    //LOGGER.info(body);
                var postURI = URI.create(auctionHouseUri + "/bid");
                HttpRequest postRequest = HttpRequest.newBuilder()
                    .uri(postURI)
                    .header("Content-Type", BidJsonRepresentation.MEDIA_TYPE)
                    .POST(HttpRequest.BodyPublishers.ofString(body))
                    .build();

                HttpClient client = HttpClient.newHttpClient();
                var postResponse = client.send(postRequest, HttpResponse.BodyHandlers.ofString());
                } catch (JsonProcessingException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                } catch (IOException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                } catch (InterruptedException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                }


                //LOGGER.info(postResponse.statusCode());
        };

        // for (JSONObject auction : auctions) {
            // auctionStartedHandler.handleAuctionStartedEvent(
            //     new AuctionStartedEvent(
            //         new Auction(new AuctionId(auction.getAuctionId()),
            //         new AuctionHouseUri(new URI(auction.getAuctionHouseUri())),
            //         new AuctionedTaskUri(new URI(auction.getTaskUri())),
            //         new AuctionedTaskType(auction.getTaskType()),
            //         new AuctionDeadline(auction.getDeadline()))
            //     ));
        // }

        return new ResponseEntity<>(HttpStatus.OK);
    }
}
