package ch.unisg.tapas.auctionhouse.adapter.out.messaging.websub;

import ch.unisg.tapas.auctionhouse.application.port.out.AuctionStartedEventPort;
import ch.unisg.tapas.auctionhouse.domain.AuctionStartedEvent;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Primary;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.net.URI;
import java.net.http.HttpClient;
import java.net.http.HttpRequest;
import java.net.http.HttpResponse;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * This class is a template for publishing auction started events via WebSub.
 */
@Component
// @Primary
public class PublishAuctionStartedEventWebSubAdapter implements AuctionStartedEventPort {
    @Value("${auction.house.uri}")
    private String auctionHouseUri;

    @Value("${websub.hub.uri}")
    private String webSubHubUri;

    Logger logger = Logger.getLogger(PublishAuctionStartedEventWebSubAdapter.class.getName());

    @Override
    public void publishAuctionStartedEvent(AuctionStartedEvent event) {
        logger.info("AuctionHouse | Publish auction started over WebSub!");
        logger.info("AuctionHouse | AuctionHouseURI: " + auctionHouseUri + " WebSubHubUri: " + webSubHubUri);

        HttpClient client = HttpClient.newHttpClient();

        String body = "hub.url=" + auctionHouseUri + "/auctions/&hub.mode=publish";

        HttpRequest request = HttpRequest.newBuilder()
                .uri(URI.create(webSubHubUri))
                .header("Content-Type", "application/x-www-form-urlencoded")
                .POST(HttpRequest.BodyPublishers.ofString(body))
                .build();


        try {
            client.send(request, HttpResponse.BodyHandlers.ofString());
            logger.info("AuctionHouse | Publish auction over WebSub successfuly!");
        } catch (InterruptedException e) {
            logger.log(Level.SEVERE, e.getLocalizedMessage(), e);
            Thread.currentThread().interrupt();
        } catch (IOException e) {
            logger.log(Level.SEVERE, e.getLocalizedMessage(), e);
        }
    }
}
