package ch.unisg.tapas.auctionhouse.adapter.out.web;

import ch.unisg.tapas.auctionhouse.domain.ExecutorInfo;
import org.apache.logging.log4j.Level;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.json.JSONArray;
import org.springframework.http.HttpStatus;

import java.io.IOException;
import java.net.URI;
import java.net.http.HttpClient;
import java.net.http.HttpRequest;
import java.net.http.HttpResponse;
import java.util.LinkedList;
import java.util.List;

public class GetExecutorsFromExecutorPoolWebAdapter {
    private static final Logger LOGGER = LogManager.getLogger(GetExecutorsFromExecutorPoolWebAdapter.class);

    // TODO get from config
    String server = "http://localhost:8083";

    public List<ExecutorInfo> getExecutorsInExecutorPool(){
        var client = HttpClient.newHttpClient();
        var request = HttpRequest.newBuilder()
            .uri(URI.create(server + "/executor-pool/GetAllExecutorsInExecutorPool"))
            .GET()
            .build();

        try {
            var response = client.send(request, HttpResponse.BodyHandlers.ofString());

            if(response.statusCode() != HttpStatus.OK.value()){
                LOGGER.log(Level.INFO, "Could not get executors from Executor Pool");
                return null;
            }

            LOGGER.log(Level.INFO, "Executors received from ExecutorPool: " + response.body());

            var jsonExecutorArray = new JSONArray(response.body());
            var executorList = new LinkedList<ExecutorInfo>();

            for(int i = 0; i < jsonExecutorArray.length(); i++){
                var jsonExecutorObject = jsonExecutorArray.getJSONObject(i);
                var executorURI = jsonExecutorObject.getString("executorUri");
                var executorType = jsonExecutorObject.getString("executorTaskType");
                executorList.add(new ExecutorInfo(executorURI, executorType));
            }

            return executorList;
        } catch (IOException e) {
            e.printStackTrace();
        } catch (InterruptedException e) {
            e.printStackTrace();
        }

        return null;
    }
}
