package ch.unisg.tapastasks.tasks.adapter.in.web;

import ch.unisg.tapastasks.tasks.adapter.in.formats.TaskJsonRepresentation;
import ch.unisg.tapastasks.tasks.application.port.in.CompleteTaskCommand;
import ch.unisg.tapastasks.tasks.application.port.in.CompleteTaskUseCase;
import ch.unisg.tapastasks.tasks.domain.Task;
import com.fasterxml.jackson.core.JsonProcessingException;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.server.ResponseStatusException;

import java.util.logging.Logger;

import javax.validation.ConstraintViolationException;

@RestController
public class CompleteTaskWebController {
    private final CompleteTaskUseCase completeTaskUseCase;

    Logger logger = Logger.getLogger(CompleteTaskWebController.class.getName());

    public CompleteTaskWebController(CompleteTaskUseCase completeTaskUseCase){
        this.completeTaskUseCase = completeTaskUseCase;
    }

    @PostMapping(path = "/tasks/completeTask")
    public ResponseEntity<String> completeTask (@RequestBody TaskJsonRepresentation payload) {
        logger.info("TaskList | Complete task request.");

        try {
            CompleteTaskCommand command = new CompleteTaskCommand(
                new Task.TaskId(payload.getTaskId()), new Task.OutputData(payload.getOutputData())
            );

            Task updateATask = completeTaskUseCase.completeTask(command);

            HttpHeaders responseHeaders = new HttpHeaders();
            responseHeaders.add(HttpHeaders.CONTENT_TYPE, TaskJsonRepresentation.MEDIA_TYPE);

            logger.info("TaskList | Task completed successfuly");
            return new ResponseEntity<>(TaskJsonRepresentation.serialize(updateATask), responseHeaders, HttpStatus.ACCEPTED);
        } catch (JsonProcessingException e) {
            throw new ResponseStatusException(HttpStatus.INTERNAL_SERVER_ERROR, e.getMessage());
        } catch (ConstraintViolationException e) {
            throw new ResponseStatusException(HttpStatus.BAD_REQUEST, e.getMessage());
        }
    }
}
