package ch.unisg.tapastasks.tasks.adapter.out.web;


import ch.unisg.tapastasks.tasks.application.port.out.CanTaskBeDeletedPort;
import ch.unisg.tapastasks.tasks.domain.DeleteTaskEvent;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Primary;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.net.URI;
import java.net.http.HttpClient;
import java.net.http.HttpRequest;
import java.net.http.HttpResponse;
import java.util.logging.Level;
import java.util.logging.Logger;

@Component
@Primary
public class CanTaskBeDeletedWebAdapter implements CanTaskBeDeletedPort {

    Logger logger = Logger.getLogger(CanTaskBeDeletedWebAdapter.class.getName());

    @Value("${roster.uri}")
    String server;

    @Override
    public boolean canTaskBeDeletedEvent(DeleteTaskEvent event){
        HttpClient client = HttpClient.newHttpClient();
        HttpRequest request = HttpRequest.newBuilder()
            .uri(URI.create(server + "/task/" + event.taskId))
            .header("Content-Type", "application/task+json")
            .DELETE()
            .build();

        try {
            HttpResponse<String> response = client.send(request, HttpResponse.BodyHandlers.ofString());
            return response.statusCode() == HttpStatus.OK.value();
        } catch (IOException e){
            logger.log(Level.SEVERE, e.getLocalizedMessage(), e);
        } catch (InterruptedException e) {
            logger.log(Level.SEVERE, e.getLocalizedMessage(), e);
            Thread.currentThread().interrupt();
        }

        return false;
    }
}
