package ch.unisg.tapastasks.tasks.adapter.out.web;

import ch.unisg.tapastasks.tasks.application.port.out.NewTaskAddedEventPort;
import ch.unisg.tapastasks.tasks.domain.NewTaskAddedEvent;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Primary;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.net.URI;
import java.net.http.HttpClient;
import java.net.http.HttpRequest;
import java.net.http.HttpResponse;
import java.util.HashMap;
import java.util.logging.Level;
import java.util.logging.Logger;

@Component
@Primary
public class PublishNewTaskAddedEventWebAdapter implements NewTaskAddedEventPort {

    Logger logger = Logger.getLogger(PublishNewTaskAddedEventWebAdapter.class.getName());

    @Value("${roster.uri}")
    String server;

    @Value("${baseuri}")
    String baseuri;

    @Override
    public void publishNewTaskAddedEvent(NewTaskAddedEvent event) {

        // Here we would need to work with DTOs in case the payload of calls becomes more complex

        var values = new HashMap<String, String>();
        values.put("taskID", event.taskId);
        values.put("taskType", event.taskType);
        values.put("inputData", event.inputData);

        var objectMapper = new ObjectMapper();
        String requestBody = null;
        try {
            requestBody = objectMapper.writeValueAsString(values);
        } catch (JsonProcessingException e) {
            e.printStackTrace();
        }

        HttpClient client = HttpClient.newHttpClient();
        HttpRequest request = HttpRequest.newBuilder()
                .uri(URI.create(server+"/task"))
                .header("Content-Type", "application/task+json")
                .header("Link", "<" + baseuri + "tasks/" + event.taskId + ">; rel=\"task\"")
                .POST(HttpRequest.BodyPublishers.ofString(requestBody))
                .build();

        try {
            client.send(request, HttpResponse.BodyHandlers.ofString());
        } catch (IOException e) {
            e.printStackTrace();
        } catch (InterruptedException e) {
            logger.log(Level.SEVERE, e.getLocalizedMessage(), e);
            Thread.currentThread().interrupt();
        }
    }
}
